<?php
namespace tests\Zeedhi\Zmart\QueryBuilders\Mongo;

use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Zmart\DataSource\AggregateFunction;
use Zeedhi\Zmart\Util\MathExpression;
use Zeedhi\Zmart\DataSource\ZmartFilterCriteria;
use Zeedhi\Zmart\QueryBuilders\Mongo\QueryBuilder;

class QueryBuilderTest extends \PHPUnit_Framework_TestCase {

    /**
     * @var QueryBuilder
     */
    protected $queryBuilder;

    public function setUp() {
        $this->queryBuilder = new QueryBuilder();
    }

    public function testEmptyQuery() {
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 0);
        $query = $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
        $expectedQuery = array();
        $this->assertEquals($expectedQuery, $query);
    }

    public function testLimit () {
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 50);
        $query = $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
        $expectedQuery = array(array('$limit' => 50));
        $this->assertEquals($expectedQuery, $query);
    }

    public function testSkipLimit() {
        $filterCriteria = new ZmartFilterCriteria("name", array(), 2, 50);
        $query = $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
        $expectedQuery = array(array('$skip' => 50), array('$limit' => 50));
        $this->assertEquals($expectedQuery, $query);
    }

    public function testSkipNoLimit() {
        $filterCriteria = new ZmartFilterCriteria("name", array(), 2, 0);
        $query = $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
        $expectedQuery = array();
        $this->assertEquals($expectedQuery, $query);
    }

    public function testSkipFirstPage() {
        $filterCriteria = new ZmartFilterCriteria("name", array(), 1, 50);
        $query = $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
        $expectedQuery = array(array('$limit' => 50));
        $this->assertEquals($expectedQuery, $query);
    }

    public function testSortByDescendant() {
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 0);
        $filterCriteria->addOrderBy("_id", ZmartFilterCriteria::ORDER_DESC);
        $query = $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
        $expectedQuery = array(array('$sort' => array('_id' => -1)));
        $this->assertEquals($expectedQuery, $query);
    }

    public function testSortByAscendant() {
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 0);
        $filterCriteria->addOrderBy("_id", ZmartFilterCriteria::ORDER_ASC);
        $query = $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
        $expectedQuery = array(array('$sort' => array('_id' => 1)));
        $this->assertEquals($expectedQuery, $query);
    }

    public function testSortByIdDescendant() {
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 0);
        $filterCriteria->addOrderBy('branch', ZmartFilterCriteria::ORDER_DESC);
        $filterCriteria->addGroupBy('branch');
        $query = $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
        $expectedQuery = array(array('$group' => array('_id' => '$branch')), array('$sort' => array('_id' => -1)));
        $this->assertEquals($expectedQuery, $query);
    }

    public function testSortByIdProperty() {
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 0);
        $filterCriteria->addOrderBy('branch', ZmartFilterCriteria::ORDER_DESC);
        $filterCriteria->addGroupBy('branch');
        $filterCriteria->addGroupBy('pow2');
        $query = $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
        $expectedQuery = array(array('$group' => array('_id' => array('branch' => '$branch', 'pow2' => '$pow2'))), array('$sort' => array('_id.branch' => -1)));
        $this->assertEquals($expectedQuery, $query);
    }

    public function testSortByTwoColumns() {
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 0);
        $filterCriteria->addOrderBy("_id", ZmartFilterCriteria::ORDER_DESC);
        $filterCriteria->addOrderBy("name", ZmartFilterCriteria::ORDER_ASC);
        $query = $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
        $expectedQuery = array(array('$sort' => array('_id' => -1, 'name' => 1)));
        $this->assertEquals($expectedQuery, $query);
    }

    public function testInvalidSort() {
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 0);
        $filterCriteria->addOrderBy("_id", 2);
        $this->setExpectedException('\Zeedhi\Zmart\QueryBuilders\Mongo\Exception', "Invalid Sort Direction 2. Accepted sort directions 1 or ".FilterCriteria::ORDER_ASC." to ascendant, -1 or ".FilterCriteria::ORDER_DESC." to descendent");
        $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
    }

    public function testGetSortAscendant() {
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 0);
        $filterCriteria->addOrderBy("_id", ZmartFilterCriteria::ORDER_ASC);
        $query = $this->queryBuilder->getSort($filterCriteria->getOrderBy());
        $expectedQuery = array('_id' => 1);
        $this->assertEquals($expectedQuery, $query);
    }

    public function testGetSortDescendant() {
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 0);
        $filterCriteria->addOrderBy("_id", ZmartFilterCriteria::ORDER_DESC);
        $query = $this->queryBuilder->getSort($filterCriteria->getOrderBy());
        $expectedQuery = array('_id' => -1);
        $this->assertEquals($expectedQuery, $query);
    }

    public function testGetEmptySort() {
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 0);
        $query = $this->queryBuilder->getSort($filterCriteria->getOrderBy());
        $this->assertNull($query);
    }

    public function testGetTwoSorts() {
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 0);
        $filterCriteria->addOrderBy("_id", ZmartFilterCriteria::ORDER_DESC);
        $filterCriteria->addOrderBy("name", ZmartFilterCriteria::ORDER_ASC);
        $query = $this->queryBuilder->getSort($filterCriteria->getOrderBy());
        $expectedQuery = array('_id' => -1, 'name' => 1);
        $this->assertEquals($expectedQuery, $query);
    }

    public function testGetInvalidSort() {
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 0);
        $filterCriteria->addOrderBy("_id", 2);
        $this->setExpectedException('\Zeedhi\Zmart\QueryBuilders\Mongo\Exception', "Invalid Sort Direction 2. Accepted sort directions 1 or ".FilterCriteria::ORDER_ASC." to ascendant, -1 or ".FilterCriteria::ORDER_DESC." to descendent");
        $this->queryBuilder->getSort($filterCriteria->getOrderBy());
    }

    public function testLimitAndSortBy() {
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 50);
        $filterCriteria->addOrderBy("_id", ZmartFilterCriteria::ORDER_DESC);
        $query = $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
        $expectedQuery = array(array('$sort' => array('_id' => -1)), array('$limit' => 50));
        $this->assertEquals($expectedQuery, $query);
    }

    public function testPostEquals() {
        $conditions = array(
            array(
                'columnName' => 'id',
                'operator' => ZmartFilterCriteria::EQ,
                'value' => 32
            )
        );
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 0);
        $filterCriteria->setPostConditions($conditions);
        $query = $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
        $expectedQuery = array(array('$match' => array('id' => array('$eq' => 32))));
        $this->assertEquals($expectedQuery, $query);
    }

    public function testPostNotEquals() {
        $conditions = array(
            array(
                'columnName' => 'id',
                'operator' => ZmartFilterCriteria::NEQ,
                'value' => 32
            )
        );
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 0);
        $filterCriteria->setPostConditions($conditions);
        $query = $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
        $expectedQuery = array(array('$match' => array('id' => array('$ne' => 32))));
        $this->assertEquals($expectedQuery, $query);
    }

    public function testPostreater() {
        $conditions = array(
            array(
                'columnName' => 'id',
                'operator' => ZmartFilterCriteria::GT,
                'value' => 32
            )
        );
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 0);
        $filterCriteria->setPostConditions($conditions);
        $query = $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
        $expectedQuery = array(array('$match' => array('id' => array('$gt' => 32))));
        $this->assertEquals($expectedQuery, $query);
    }

    public function testPostGreaterOrEqual() {
        $conditions = array(
            array(
                'columnName' => 'id',
                'operator' => ZmartFilterCriteria::GTE,
                'value' => 32
            )
        );
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 0);
        $filterCriteria->setPostConditions($conditions);
        $query = $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
        $expectedQuery = array(array('$match' => array('id' => array('$gte' => 32))));
        $this->assertEquals($expectedQuery, $query);
    }

    public function testPostLower() {
        $conditions = array(
            array(
                'columnName' => 'id',
                'operator' => ZmartFilterCriteria::LT,
                'value' => 32
            )
        );
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 0);
        $filterCriteria->setPostConditions($conditions);
        $query = $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
        $expectedQuery = array(array('$match' => array('id' => array('$lt' => 32))));
        $this->assertEquals($expectedQuery, $query);
    }

    public function testPostLowerOrEqual() {
        $conditions = array(
            array(
                'columnName' => 'id',
                'operator' => ZmartFilterCriteria::LTE,
                'value' => 32
            )
        );
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 0);
        $filterCriteria->setPostConditions($conditions);
        $query = $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
        $expectedQuery = array(array('$match' => array('id' => array('$lte' => 32))));
        $this->assertEquals($expectedQuery, $query);
    }

    public function testPostIn() {
        $conditions = array(
            array(
                'columnName' => 'id',
                'operator' => ZmartFilterCriteria::IN,
                'value' => [32, 64]
            )
        );
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 0);
        $filterCriteria->setPostConditions($conditions);
        $query = $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
        $expectedQuery = array(array('$match' => array('id' => array('$in' => array(32, 64)))));
        $this->assertEquals($expectedQuery, $query);
    }

    public function testPostNotIn() {
        $conditions = array(
            array(
                'columnName' => 'id',
                'operator' => ZmartFilterCriteria::NOT_IN,
                'value' => [32, 64]
            )
        );
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 0);
        $filterCriteria->setPostConditions($conditions);
        $query = $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
        $expectedQuery = array(array('$match' => array('id' => array('$nin' => array(32, 64)))));
        $this->assertEquals($expectedQuery, $query);
    }

    public function testPostIsEqual() {
        $conditions = array(
            array(
                'columnName' => 'id',
                'operator' => ZmartFilterCriteria::IS,
                'value' => 32
            )
        );
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 0);
        $filterCriteria->setPostConditions($conditions);
        $query = $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
        $expectedQuery = array(array('$match' => array('id' => array('$eq' => 32))));
        $this->assertEquals($expectedQuery, $query);
    }

    public function testPostBetween() {
        $startDate = new \DateTime();
        $endDate = (new \DateTime())->setDate(2050, 10, 15);
        $conditions = array(
            array(
                'columnName' => 'id',
                'operator' => ZmartFilterCriteria::BETWEEN,
                'value' => array($startDate, $endDate)
            )
        );
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 0);
        $filterCriteria->setPostConditions($conditions);
        $query = $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
        $expectedQuery = array(array('$match' => array('id' => array('$gte' => $startDate, '$lte' => $endDate))));
        $this->assertEquals($expectedQuery, $query);
    }

    public function testInvalidBetween() {
        $conditions = array(
            array(
                'columnName' => 'id',
                'operator' => ZmartFilterCriteria::BETWEEN,
                'value' => array(200)
            )
        );
        $filterCriteria = new ZmartFilterCriteria("name", $conditions, null, 0);
        $exceptionMessage = "Invalid between value. It should be an array with first key as the lower value and second as the greater. Given value was ";
        $this->setExpectedException('\Zeedhi\Zmart\QueryBuilders\Mongo\Exception', $exceptionMessage);
        $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
    }

    public function testPostTwoColuns() {
        $conditions = array(
            array(
                'columnName' => 'id',
                'operator' => ZmartFilterCriteria::EQ,
                'value' => 32
            ),
            array(
                'columnName' => 'desc',
                'operator' => ZmartFilterCriteria::EQ,
                'value' => 'thirty two'
            )
        );
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 0);
        $filterCriteria->setPostConditions($conditions);
        $query = $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
        $expectedQuery = array(array('$match' => array('id' => array('$eq' => 32), 'desc' => array('$eq' => 'thirty two'))));
        $this->assertEquals($expectedQuery, $query);
    }

    public function testPostDiferentOperations() {
        $conditions = array(
            array(
                'columnName' => 'id',
                'operator' => ZmartFilterCriteria::EQ,
                'value' => 32
            ),
            array(
                'columnName' => 'id',
                'operator' => ZmartFilterCriteria::GT,
                'value' => 31
            )
        );
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 0);
        $filterCriteria->setPostConditions($conditions);
        $query = $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
        $expectedQuery = array(array('$match' => array('id' => array('$gt' => 31))));
        $this->assertEquals($expectedQuery, $query);
    }

    public function testPostDiferentOperationsAndDiferentFields() {
        $conditions = array(
            array(
                'columnName' => 'id',
                'operator' => ZmartFilterCriteria::GT,
                'value' => 31
            ),
            array(
                'columnName' => 'desc',
                'operator' => ZmartFilterCriteria::EQ,
                'value' => 'sixty four'
            )
        );
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 0);
        $filterCriteria->setPostConditions($conditions);
        $query = $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
        $expectedQuery = array(array('$match' => array('id' => array('$gt' => 31), 'desc' => array('$eq' => 'sixty four'))));
        $this->assertEquals($expectedQuery, $query);
    }

    public function testPostThreeOperations() {
        $conditions = array(
            array(
                'columnName' => 'id',
                'operator' => ZmartFilterCriteria::GT,
                'value' => 31
            ),
            array(
                'columnName' => 'desc',
                'operator' => ZmartFilterCriteria::EQ,
                'value' => 'sixty four'
            ),
            array(
                'columnName' => 'pow2',
                'operator' => ZmartFilterCriteria::EQ,
                'value' => 128
            )
        );
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 0);
        $filterCriteria->setPostConditions($conditions);
        $query = $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
        $expectedQuery = array(
            array(
                '$match' => array(
                    'id' => array(
                        '$gt' => 31
                    ),
                    'desc' => array(
                        '$eq' => 'sixty four'
                    ),
                    'pow2' => array(
                        '$eq' => 128
                    )
                )
            )
        );
        $this->assertEquals($expectedQuery, $query);
    }


    public function testInvalidOperator() {
        $conditions = array(
            array(
                'columnName' => 'id',
                'operator' => 'Ingual',
                'value' => 32
            )
        );
        $filterCriteria = new ZmartFilterCriteria("name", $conditions, null, 0);
        $exceptionMessage = "Invalid operator Ingual. Operators accepted: ";
        $operatorAccepted = implode(', ', array_keys(QueryBuilder::$OPERATORS_COMPARISON));
        $exceptionMessage .= $operatorAccepted;
        $this->setExpectedException('\Zeedhi\Zmart\QueryBuilders\Mongo\Exception', $exceptionMessage);
        $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
    }

    public function testGetPostConditionsEquals() {
        $conditions = array(
            array(
                'columnName' => 'id',
                'operator' => ZmartFilterCriteria::EQ,
                'value' => 32
            )
        );
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 0);
        $filterCriteria->setPostConditions($conditions);
        $query = $this->queryBuilder->getWhere($filterCriteria->getPostConditions());
        $expectedQuery = array('id' => array('$eq' => 32));
        $this->assertEquals($expectedQuery, $query);
    }

    public function testGetPostConditionsNotEquals() {
        $conditions = array(
            array(
                'columnName' => 'id',
                'operator' => ZmartFilterCriteria::NEQ,
                'value' => 32
            )
        );
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 0);
        $filterCriteria->setPostConditions($conditions);
        $query = $this->queryBuilder->getWhere($filterCriteria->getPostConditions());
        $expectedQuery = array('id' => array('$ne' => 32));
        $this->assertEquals($expectedQuery, $query);
    }

    public function testGetPostGreater() {
        $conditions = array(
            array(
                'columnName' => 'id',
                'operator' => ZmartFilterCriteria::GT,
                'value' => 32
            )
        );
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 0);
        $filterCriteria->setPostConditions($conditions);
        $query = $this->queryBuilder->getWhere($filterCriteria->getPostConditions());
        $expectedQuery = array('id' => array('$gt' => 32));
        $this->assertEquals($expectedQuery, $query);
    }

    public function testGetPostGreaterOrEqual() {
        $conditions = array(
            array(
                'columnName' => 'id',
                'operator' => ZmartFilterCriteria::GTE,
                'value' => 32
            )
        );
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 0);
        $filterCriteria->setPostConditions($conditions);
        $query = $this->queryBuilder->getWhere($filterCriteria->getPostConditions());
        $expectedQuery = array('id' => array('$gte' => 32));
        $this->assertEquals($expectedQuery, $query);
    }

    public function testGetPostLower() {
        $conditions = array(
            array(
                'columnName' => 'id',
                'operator' => ZmartFilterCriteria::LT,
                'value' => 32
            )
        );
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 0);
        $filterCriteria->setPostConditions($conditions);
        $query = $this->queryBuilder->getWhere($filterCriteria->getPostConditions());
        $expectedQuery = array('id' => array('$lt' => 32));
        $this->assertEquals($expectedQuery, $query);
    }

    public function testGetPostLowerOrEqual() {
        $conditions = array(
            array(
                'columnName' => 'id',
                'operator' => ZmartFilterCriteria::LTE,
                'value' => 32
            )
        );
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 0);
        $filterCriteria->setPostConditions($conditions);
        $query = $this->queryBuilder->getWhere($filterCriteria->getPostConditions());
        $expectedQuery = array('id' => array('$lte' => 32));
        $this->assertEquals($expectedQuery, $query);
    }

    public function testGetPostIn() {
        $conditions = array(
            array(
                'columnName' => 'id',
                'operator' => ZmartFilterCriteria::IN,
                'value' => [32, 64]
            )
        );
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 0);
        $filterCriteria->setPostConditions($conditions);
        $query = $this->queryBuilder->getWhere($filterCriteria->getPostConditions());
        $expectedQuery = array('id' => array('$in' => array(32, 64)));
        $this->assertEquals($expectedQuery, $query);
    }

    public function testGetPostNotIn() {
        $conditions = array(
            array(
                'columnName' => 'id',
                'operator' => ZmartFilterCriteria::NOT_IN,
                'value' => [32, 64]
            )
        );
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 0);
        $filterCriteria->setPostConditions($conditions);
        $query = $this->queryBuilder->getWhere($filterCriteria->getPostConditions());
        $expectedQuery = array('id' => array('$nin' => array(32, 64)));
        $this->assertEquals($expectedQuery, $query);
    }

    public function testGetPostIsEqual() {
        $conditions = array(
            array(
                'columnName' => 'id',
                'operator' => ZmartFilterCriteria::IS,
                'value' => 32
            )
        );
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 0);
        $filterCriteria->setPostConditions($conditions);
        $query = $this->queryBuilder->getWhere($filterCriteria->getPostConditions());
        $expectedQuery = array('id' => array('$eq' => 32));
        $this->assertEquals($expectedQuery, $query);
    }

    public function testGetPostBetween() {
        $startDate = new \DateTime();
        $endDate = (new \DateTime())->setDate(2050, 10, 15);
        $conditions = array(
            array(
                'columnName' => 'id',
                'operator' => ZmartFilterCriteria::BETWEEN,
                'value' => array($startDate, $endDate)
            )
        );
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 0);
        $filterCriteria->setPostConditions($conditions);
        $query = $this->queryBuilder->getWhere($filterCriteria->getPostConditions());
        $expectedQuery = array('id' => array('$gte' => $startDate, '$lte' => $endDate));
        $this->assertEquals($expectedQuery, $query);
    }

    public function testGetInvalidBetween() {
        $conditions = array(
            array(
                'columnName' => 'id',
                'operator' => ZmartFilterCriteria::BETWEEN,
                'value' => array(200)
            )
        );
        $filterCriteria = new ZmartFilterCriteria("name", $conditions, null, 0);
        $exceptionMessage = "Invalid between value. It should be an array with first key as the lower value and second as the greater. Given value was ";
        $this->setExpectedException('\Zeedhi\Zmart\QueryBuilders\Mongo\Exception', $exceptionMessage);
        $this->queryBuilder->getWhere($filterCriteria->getConditions());
    }

    public function testGetPostTwoColuns() {
        $conditions = array(
            array(
                'columnName' => 'id',
                'operator' => ZmartFilterCriteria::EQ,
                'value' => 32
            ),
            array(
                'columnName' => 'desc',
                'operator' => ZmartFilterCriteria::EQ,
                'value' => 'thirty two'
            )
        );
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 0);
        $filterCriteria->setPostConditions($conditions);
        $query = $this->queryBuilder->getWhere($filterCriteria->getPostConditions());
        $expectedQuery = array('id' => array('$eq' => 32), 'desc' => array('$eq' => 'thirty two'));
        $this->assertEquals($expectedQuery, $query);
    }

    public function testGetPostDiferentOperations() {
        $conditions = array(
            array(
                'columnName' => 'id',
                'operator' => ZmartFilterCriteria::EQ,
                'value' => 32
            ),
            array(
                'columnName' => 'id',
                'operator' => ZmartFilterCriteria::GT,
                'value' => 31
            )
        );
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 0);
        $filterCriteria->setPostConditions($conditions);
        $query = $this->queryBuilder->getWhere($filterCriteria->getPostConditions());
        $expectedQuery = array('id' => array('$gt' => 31));
        $this->assertEquals($expectedQuery, $query);
    }

    public function testGetPostDiferentOperationsAndDiferentFields() {
        $conditions = array(
            array(
                'columnName' => 'id',
                'operator' => ZmartFilterCriteria::GT,
                'value' => 31
            ),
            array(
                'columnName' => 'desc',
                'operator' => ZmartFilterCriteria::EQ,
                'value' => 'sixty four'
            )
        );
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 0);
        $filterCriteria->setPostConditions($conditions);
        $query = $this->queryBuilder->getWhere($filterCriteria->getPostConditions());
        $expectedQuery = array('id' => array('$gt' => 31), 'desc' => array('$eq' => 'sixty four'));
        $this->assertEquals($expectedQuery, $query);
    }

    public function testGetPostThreeOperations() {
        $conditions = array(
            array(
                'columnName' => 'id',
                'operator' => ZmartFilterCriteria::GT,
                'value' => 31
            ),
            array(
                'columnName' => 'desc',
                'operator' => ZmartFilterCriteria::EQ,
                'value' => 'sixty four'
            ),
            array(
                'columnName' => 'pow2',
                'operator' => ZmartFilterCriteria::EQ,
                'value' => 128
            )
        );
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 0);
        $filterCriteria->setPostConditions($conditions);
        $query = $this->queryBuilder->getWhere($filterCriteria->getPostConditions());
        $expectedQuery = array(
            'id' => array(
                '$gt' => 31
            ),
            'desc' => array(
                '$eq' => 'sixty four'
            ),
            'pow2' => array(
                '$eq' => 128
            )
        );
        $this->assertEquals($expectedQuery, $query);
    }

    public function testGetPostEmpty() {
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 0);
        $query = $this->queryBuilder->getWhere($filterCriteria->getPostConditions());
        $this->assertNull($query);
    }

    public function testGetInvalidOperator() {
        $conditions = array(
            array(
                'columnName' => 'id',
                'operator' => 'Ingual',
                'value' => 32
            )
        );
        $filterCriteria = new ZmartFilterCriteria("name", array(0), null, 0);
        $filterCriteria->setPostConditions($conditions);
        $exceptionMessage = "Invalid operator Ingual. Operators accepted: ";
        $operatorAccepted = implode(', ', array_keys(QueryBuilder::$OPERATORS_COMPARISON));
        $exceptionMessage .= $operatorAccepted;
        $this->setExpectedException('\Zeedhi\Zmart\QueryBuilders\Mongo\Exception', $exceptionMessage);
        $this->queryBuilder->getWhere($filterCriteria->getPostConditions());
    }

    public function testPostLimitAndSort() {
        $conditions = array(
            array(
                'columnName' => 'id',
                'operator' => ZmartFilterCriteria::GT,
                'value' => 31
            ),
            array(
                'columnName' => 'desc',
                'operator' => ZmartFilterCriteria::EQ,
                'value' => 'sixty four'
            ),
            array(
                'columnName' => 'pow2',
                'operator' => ZmartFilterCriteria::EQ,
                'value' => 128
            )
        );
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 50);
        $filterCriteria->setPostConditions($conditions);
        $filterCriteria->addOrderBy("id", ZmartFilterCriteria::ORDER_DESC);
        $filterCriteria->addOrderBy("name", ZmartFilterCriteria::ORDER_ASC);
        $query = $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
        $expectedQuery = array(
            array(
                '$match' => array(
                    'id' => array(
                        '$gt' => 31),
                    'desc' => array(
                        '$eq' => 'sixty four'
                    ),
                    'pow2' => array(
                        '$eq' => 128
                    )
                )
            ),
            array(
                '$sort' => array(
                    'id' => -1,
                    'name' => 1
                )
            ),
            array(
                '$limit' => 50
            )
        );
        $this->assertEquals($expectedQuery, $query);
    }

    public function testGroupBy() {
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 0);
        $filterCriteria->addGroupBy('price');
        $query = $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
        $expectedQuery = array(array('$group' => array('_id' => '$price')));
        $this->assertEquals($expectedQuery, $query);
    }

    public function testGroupByTwoFields() {
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 0);
        $filterCriteria->addGroupBy('price');
        $filterCriteria->addGroupBy('brand');
        $query = $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
        $expectedQuery = array(array('$group' => array('_id' => array('price' => '$price', 'brand' => '$brand'))));
        $this->assertEquals($expectedQuery, $query);
    }

    public function testGroupByThreeFields() {
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 0);
        $filterCriteria->setGroupBy(array('price', 'brand', 'store'));
        $query = $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
        $expectedQuery = array(
            array(
                '$group' => array(
                    '_id' => array(
                        'price' => '$price',
                        'brand' => '$brand',
                        'store' => '$store'
                    )
                )
            )
        );
        $this->assertEquals($expectedQuery, $query);
    }

    public function testGroupPostSortAndLimit() {
        $conditions = array(
            array(
                'columnName' => 'id',
                'operator' => ZmartFilterCriteria::GT,
                'value' => 31
            ),
            array(
                'columnName' => 'desc',
                'operator' => ZmartFilterCriteria::EQ,
                'value' => 'sixty four'
            ),
            array(
                'columnName' => 'pow2',
                'operator' => ZmartFilterCriteria::EQ,
                'value' => 128
            )
        );
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 50);
        $filterCriteria->setPostConditions($conditions);
        $filterCriteria->addOrderBy("id", ZmartFilterCriteria::ORDER_DESC);
        $filterCriteria->addOrderBy("name", ZmartFilterCriteria::ORDER_ASC);
        $filterCriteria->addGroupBy('price');
        $filterCriteria->addGroupBy('brand');
        $filterCriteria->addGroupBy('store');
        $query = $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
        $expectedQuery = array(
            array(
                '$group' => array(
                    '_id' => array(
                        'price' => '$price',
                        'brand' => '$brand',
                        'store' => '$store'
                    )
                )
            ),
            array(
                '$match' => array(
                    'id' => array(
                        '$gt' => 31),
                    'desc' => array(
                        '$eq' => 'sixty four'
                    ),
                    'pow2' => array(
                        '$eq' => 128
                    )
                )
            ),
            array(
                '$sort' => array(
                    'id' => -1,
                    'name' => 1
                )
            ),
            array(
                '$limit' => 50
            )
        );
        $this->assertEquals($expectedQuery, $query);
    }

    public function testAggregateFunctionSum() {
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 0);
        $filterCriteria->addGroupBy('brand');
        $filterCriteria->addAggregateFunction("BRAND_SUM_PRICE", ZmartFilterCriteria::SUM, 'price');
        $query = $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
        $expectedQuery = array(array('$group' => array('_id' => '$brand', 'BRAND_SUM_PRICE' => array('$sum' => '$price'))));
        $this->assertEquals($expectedQuery, $query);
    }

    public function testAggregateFunctionAvg() {
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 0);
        $filterCriteria->addGroupBy('brand');
        $filterCriteria->addAggregateFunction("BRAND_AVG_PRICE", ZmartFilterCriteria::AVG, 'price');
        $query = $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
        $expectedQuery = array(array('$group' => array('_id' => '$brand', 'BRAND_AVG_PRICE' => array('$avg' => '$price'))));
        $this->assertEquals($expectedQuery, $query);
    }

    public function testAggregateFunctionMin() {
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 0);
        $filterCriteria->addGroupBy('brand');
        $filterCriteria->addAggregateFunction("BRAND_MIN_PRICE", ZmartFilterCriteria::MIN, 'price');
        $query = $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
        $expectedQuery = array(array('$group' => array('_id' => '$brand', 'BRAND_MIN_PRICE' => array('$min' => '$price'))));
        $this->assertEquals($expectedQuery, $query);
    }

    public function testAggregateFunctionMax() {
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 0);
        $filterCriteria->addGroupBy('brand');
        $filterCriteria->addAggregateFunction("BRAND_MAX_PRICE", ZmartFilterCriteria::MAX, 'price');
        $query = $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
        $expectedQuery = array(array('$group' => array('_id' => '$brand', 'BRAND_MAX_PRICE' => array('$max' => '$price'))));
        $this->assertEquals($expectedQuery, $query);
    }

    public function testAggregateFunctionCount() {
    $filterCriteria = new ZmartFilterCriteria("name", array(), null, 0);
    $filterCriteria->addGroupBy('brand');
    $aggregateFunction = array(
        new AggregateFunction("BRAND_COUNT_PRICES", ZmartFilterCriteria::COUNT, 'price')
    );
    $filterCriteria->setAggregateFunctions($aggregateFunction);
    $query = $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
    $expectedQuery = array(array('$group' => array('_id' => '$brand', 'BRAND_COUNT_PRICES' => array('$sum' => 1))));
    $this->assertEquals($expectedQuery, $query);
    }

    public function testAggregateInvalidFunction() {
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 0);
        $filterCriteria->addGroupBy('brand');
        $filterCriteria->addAggregateFunction("BRAND_XABLAU_PRICE", 'XABLAU', 'price');
        $exceptionMessage = "Invalid function XABLAU. Functions accepted: ";
        $allFunctions = array_merge(array_keys(QueryBuilder::$AGGREGATE_FUNCTIONS), array_keys(QueryBuilder::$AGGREGATE_SPECIAL_FUNCTIONS));
        $functionsAccepted = implode(', ',$allFunctions );
        $exceptionMessage .= $functionsAccepted;
        $this->setExpectedException('\Zeedhi\Zmart\QueryBuilders\Mongo\Exception', $exceptionMessage);
        $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
    }

    public function testAggregateTwoFunctions() {
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 0);
        $filterCriteria->addGroupBy('brand');
        $filterCriteria->addAggregateFunction("BRAND_SUM_PRICE", ZmartFilterCriteria::SUM, 'price');
        $filterCriteria->addAggregateFunction("BRAND_AVG_PRICE", ZmartFilterCriteria::AVG, 'price');
        $query = $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
        $expectedQuery = array(array('$group' => array('_id' => '$brand', 'BRAND_SUM_PRICE' => array('$sum' => '$price'), 'BRAND_AVG_PRICE' => array('$avg' => '$price'))));
        $this->assertEquals($expectedQuery, $query);
    }

    public function testAggregateThreeFunction() {
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 0);
        $filterCriteria->addGroupBy('brand');
        $filterCriteria->addAggregateFunction("BRAND_AVG_PRICE", ZmartFilterCriteria::AVG, 'price');
        $filterCriteria->addAggregateFunction("BRAND_MIN_PRICE", ZmartFilterCriteria::MIN, 'price');
        $filterCriteria->addAggregateFunction("BRAND_MAX_PRICE", ZmartFilterCriteria::MAX, 'price');
        $query = $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
        $expectedQuery = array(array('$group' => array('_id' => '$brand', 'BRAND_AVG_PRICE' => array('$avg' => '$price'), 'BRAND_MIN_PRICE' => array('$min' => '$price'), 'BRAND_MAX_PRICE' => array('$max' => '$price'))));
        $this->assertEquals($expectedQuery, $query);
    }

    public function testAggregatePostGroupSortAndLimit() {
        $conditions = array(
            array(
                'columnName' => 'id',
                'operator' => ZmartFilterCriteria::GT,
                'value' => 31
            ),
            array(
                'columnName' => 'desc',
                'operator' => ZmartFilterCriteria::EQ,
                'value' => 'sixty four'
            ),
            array(
                'columnName' => 'pow2',
                'operator' => ZmartFilterCriteria::EQ,
                'value' => 128
            )
        );
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 50);
        $filterCriteria->setPostConditions($conditions);
        $filterCriteria->addGroupBy('brand');
        $filterCriteria->addGroupBy('store');
        $filterCriteria->addOrderBy("id", ZmartFilterCriteria::ORDER_DESC);
        $filterCriteria->addOrderBy("name", ZmartFilterCriteria::ORDER_ASC);
        $filterCriteria->addAggregateFunction("BRAND_AVG_PRICE", ZmartFilterCriteria::AVG, 'price');
        $filterCriteria->addAggregateFunction("BRAND_MIN_PRICE", ZmartFilterCriteria::MIN, 'price');
        $filterCriteria->addAggregateFunction("BRAND_MAX_PRICE", ZmartFilterCriteria::MAX, 'price');
        $query = $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
        $expectedQuery = array(
            array(
                '$group' => array(
                    '_id' => array(
                        'brand' => '$brand',
                        'store' => '$store'
                    ),
                    'BRAND_AVG_PRICE' => array(
                        '$avg' => '$price'
                    ),
                    'BRAND_MIN_PRICE' => array(
                        '$min' => '$price'
                    ),
                    'BRAND_MAX_PRICE' => array(
                        '$max' => '$price'
                    )
                )
            ),
            array(
                '$match' => array(
                    'id' => array(
                        '$gt' => 31),
                    'desc' => array(
                        '$eq' => 'sixty four'
                    ),
                    'pow2' => array(
                        '$eq' => 128
                    )
                )
            ),
            array(
                '$sort' => array(
                    'id' => -1,
                    'name' => 1
                )
            ),
            array(
                '$limit' => 50
            )
        );
        $this->assertEquals($expectedQuery, $query);
    }

    public function testAggregatePostGroupSortSkip() {
        $conditions = array(
            array(
                'columnName' => 'id',
                'operator' => ZmartFilterCriteria::GT,
                'value' => 31
            ),
            array(
                'columnName' => 'desc',
                'operator' => ZmartFilterCriteria::EQ,
                'value' => 'sixty four'
            ),
            array(
                'columnName' => 'pow2',
                'operator' => ZmartFilterCriteria::EQ,
                'value' => 128
            )
        );
        $filterCriteria = new ZmartFilterCriteria("name", array(), 3, 50);
        $filterCriteria->setPostConditions($conditions);
        $filterCriteria->addGroupBy('brand');
        $filterCriteria->addGroupBy('store');
        $filterCriteria->addOrderBy("id", ZmartFilterCriteria::ORDER_DESC);
        $filterCriteria->addOrderBy("name", ZmartFilterCriteria::ORDER_ASC);
        $filterCriteria->addAggregateFunction("BRAND_AVG_PRICE", ZmartFilterCriteria::AVG, 'price');
        $filterCriteria->addAggregateFunction("BRAND_MIN_PRICE", ZmartFilterCriteria::MIN, 'price');
        $filterCriteria->addAggregateFunction("BRAND_MAX_PRICE", ZmartFilterCriteria::MAX, 'price');
        $query = $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
        $expectedQuery = array(
            array(
                '$group' => array(
                    '_id' => array(
                        'brand' => '$brand',
                        'store' => '$store'
                    ),
                    'BRAND_AVG_PRICE' => array(
                        '$avg' => '$price'
                    ),
                    'BRAND_MIN_PRICE' => array(
                        '$min' => '$price'
                    ),
                    'BRAND_MAX_PRICE' => array(
                        '$max' => '$price'
                    )
                )
            ),
            array(
                '$match' => array(
                    'id' => array(
                        '$gt' => 31),
                    'desc' => array(
                        '$eq' => 'sixty four'
                    ),
                    'pow2' => array(
                        '$eq' => 128
                    )
                )
            ),
            array(
                '$sort' => array(
                    'id' => -1,
                    'name' => 1
                )
            ),
            array(
                '$skip' => 100
            ),
            array(
                '$limit' => 50
            )
        );
        $this->assertEquals($expectedQuery, $query);
    }

    public function testWhereEquals() {
        $previousConditions = array(
            array(
                'columnName' => 'id',
                'operator' => ZmartFilterCriteria::EQ,
                'value' => 32
            )
        );
        $filterCriteria = new ZmartFilterCriteria("name", $previousConditions, null, 0);
        $query = $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
        $expectedQuery = array(array('$match' => array('id' => array('$eq' => 32))));
        $this->assertEquals($expectedQuery, $query);
    }

    public function testWhereNotEquals() {
        $previousConditions = array(
            array(
                'columnName' => 'id',
                'operator' => ZmartFilterCriteria::NEQ,
                'value' => 32
            )
        );
        $filterCriteria = new ZmartFilterCriteria("name", $previousConditions, null, 0);
        $query = $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
        $expectedQuery = array(array('$match' => array('id' => array('$ne' => 32))));
        $this->assertEquals($expectedQuery, $query);
    }

    public function testWhereGreater() {
        $previousConditions = array(
            array(
                'columnName' => 'id',
                'operator' => ZmartFilterCriteria::GT,
                'value' => 32
            )
        );
        $filterCriteria = new ZmartFilterCriteria("name", $previousConditions, null, 0);
        $query = $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
        $expectedQuery = array(array('$match' => array('id' => array('$gt' => 32))));
        $this->assertEquals($expectedQuery, $query);
    }

    public function testsWhereGreaterOrEqual() {
        $previousConditions = array(
            array(
                'columnName' => 'id',
                'operator' => ZmartFilterCriteria::GTE,
                'value' => 32
            )
        );
        $filterCriteria = new ZmartFilterCriteria("name", $previousConditions, null, 0);
        $query = $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
        $expectedQuery = array(array('$match' => array('id' => array('$gte' => 32))));
        $this->assertEquals($expectedQuery, $query);
    }

    public function testWhereLower() {
        $previousConditions = array(
            array(
                'columnName' => 'id',
                'operator' => ZmartFilterCriteria::LT,
                'value' => 32
            )
        );
        $filterCriteria = new ZmartFilterCriteria("name", $previousConditions, null, 0);
        $query = $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
        $expectedQuery = array(array('$match' => array('id' => array('$lt' => 32))));
        $this->assertEquals($expectedQuery, $query);
    }

    public function testWhereLowerOrEqual() {
        $previousConditions = array(
            array(
                'columnName' => 'id',
                'operator' => ZmartFilterCriteria::LTE,
                'value' => 32
            )
        );
        $filterCriteria = new ZmartFilterCriteria("name", $previousConditions, null, 0);
        $query = $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
        $expectedQuery = array(array('$match' => array('id' => array('$lte' => 32))));
        $this->assertEquals($expectedQuery, $query);
    }

    public function testWhereIn() {
        $previousConditions = array(
            array(
                'columnName' => 'id',
                'operator' => ZmartFilterCriteria::IN,
                'value' => [32, 64]
            )
        );
        $filterCriteria = new ZmartFilterCriteria("name", $previousConditions, null, 0);
        $query = $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
        $expectedQuery = array(array('$match' => array('id' => array('$in' => array(32, 64)))));
        $this->assertEquals($expectedQuery, $query);
    }

    public function testWhereNotIn() {
        $previousConditions = array(
            array(
                'columnName' => 'id',
                'operator' => ZmartFilterCriteria::NOT_IN,
                'value' => [32, 64]
            )
        );
        $filterCriteria = new ZmartFilterCriteria("name", $previousConditions, null, 0);
        $query = $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
        $expectedQuery = array(array('$match' => array('id' => array('$nin' => array(32, 64)))));
        $this->assertEquals($expectedQuery, $query);
    }

    public function testWhereIsEqual() {
        $previousConditions = array(
            array(
                'columnName' => 'id',
                'operator' => ZmartFilterCriteria::IS,
                'value' => 32
            )
        );
        $filterCriteria = new ZmartFilterCriteria("name", $previousConditions, null, 0);
        $query = $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
        $expectedQuery = array(array('$match' => array('id' => array('$eq' => 32))));
        $this->assertEquals($expectedQuery, $query);
    }

    public function testWhereBetween() {
        $startDate = new \DateTime();
        $endDate = (new \DateTime())->setDate(2050, 10, 15);
        $previousConditions = array(
            array(
                'columnName' => 'id',
                'operator' => ZmartFilterCriteria::BETWEEN,
                'value' => array($startDate, $endDate)
            )
        );
        $filterCriteria = new ZmartFilterCriteria("name", $previousConditions, null, 0);
        $query = $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
        $expectedQuery = array(array('$match' => array('id' => array('$gte' => $startDate, '$lte' => $endDate))));
        $this->assertEquals($expectedQuery, $query);
    }

    public function testWhereInvalidBetween() {
        $previousConditions = array(
            array(
                'columnName' => 'id',
                'operator' => ZmartFilterCriteria::BETWEEN,
                'value' => array(200)
            )
        );
        $filterCriteria = new ZmartFilterCriteria("name", $previousConditions, null, 0);
        $exceptionMessage = "Invalid between value. It should be an array with first key as the lower value and second as the greater. Given value was ";
        $this->setExpectedException('\Zeedhi\Zmart\QueryBuilders\Mongo\Exception', $exceptionMessage);
        $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
    }

    public function testWhereTwoColuns() {
        $previousConditions = array(
            array(
                'columnName' => 'id',
                'operator' => ZmartFilterCriteria::EQ,
                'value' => 32
            ),
            array(
                'columnName' => 'desc',
                'operator' => ZmartFilterCriteria::EQ,
                'value' => 'thirty two'
            )
        );
        $filterCriteria = new ZmartFilterCriteria("name", $previousConditions, null, 0);
        $query = $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
        $expectedQuery = array(array('$match' => array('id' => array('$eq' => 32), 'desc' => array('$eq' => 'thirty two'))));
        $this->assertEquals($expectedQuery, $query);
    }

    public function testWhereDiferentOperations() {
        $previousConditions = array(
            array(
                'columnName' => 'id',
                'operator' => ZmartFilterCriteria::EQ,
                'value' => 32
            ),
            array(
                'columnName' => 'id',
                'operator' => ZmartFilterCriteria::GT,
                'value' => 31
            )
        );
        $filterCriteria = new ZmartFilterCriteria("name", $previousConditions, null, 0);
        $query = $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
        $expectedQuery = array(array('$match' => array('id' => array('$gt' => 31))));
        $this->assertEquals($expectedQuery, $query);
    }

    public function testWhereDiferentOperationsAndDiferentFields() {
        $previousConditions = array(
            array(
                'columnName' => 'id',
                'operator' => ZmartFilterCriteria::GT,
                'value' => 31
            ),
            array(
                'columnName' => 'desc',
                'operator' => ZmartFilterCriteria::EQ,
                'value' => 'sixty four'
            )
        );
        $filterCriteria = new ZmartFilterCriteria("name", $previousConditions, null, 0);
        $query = $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
        $expectedQuery = array(array('$match' => array('id' => array('$gt' => 31), 'desc' => array('$eq' => 'sixty four'))));
        $this->assertEquals($expectedQuery, $query);
    }

    public function testWhereThreeOperations() {
        $previousConditions = array(
            array(
                'columnName' => 'id',
                'operator' => ZmartFilterCriteria::GT,
                'value' => 31
            ),
            array(
                'columnName' => 'desc',
                'operator' => ZmartFilterCriteria::EQ,
                'value' => 'sixty four'
            ),
            array(
                'columnName' => 'pow2',
                'operator' => ZmartFilterCriteria::EQ,
                'value' => 128
            )
        );
        $filterCriteria = new ZmartFilterCriteria("name", $previousConditions, null, 0);
        $query = $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
        $expectedQuery = array(
            array(
                '$match' => array(
                    'id' => array(
                        '$gt' => 31
                    ),
                    'desc' => array(
                        '$eq' => 'sixty four'
                    ),
                    'pow2' => array(
                        '$eq' => 128
                    )
                )
            )
        );
        $this->assertEquals($expectedQuery, $query);
    }


    public function testWhereInvalidOperator() {
        $previousConditions = array(
            array(
                'columnName' => 'id',
                'operator' => 'Ingual',
                'value' => 32
            )
        );
        $filterCriteria = new ZmartFilterCriteria("name", $previousConditions, null, 0);
        $exceptionMessage = "Invalid operator Ingual. Operators accepted: ";
        $operatorAccepted = implode(', ', array_keys(QueryBuilder::$OPERATORS_COMPARISON));
        $exceptionMessage .= $operatorAccepted;
        $this->setExpectedException('\Zeedhi\Zmart\QueryBuilders\Mongo\Exception', $exceptionMessage);
        $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
    }

    public function testAllActualFunctions() { // Aggregate,Previous Where, Where, Group, Sort, Skip
        $postConditions = array(
            array(
                'columnName' => 'id',
                'operator' => ZmartFilterCriteria::GT,
                'value' => 31
            ),
            array(
                'columnName' => 'desc',
                'operator' => ZmartFilterCriteria::EQ,
                'value' => 'sixty four'
            ),
            array(
                'columnName' => 'pow2',
                'operator' => ZmartFilterCriteria::EQ,
                'value' => 128
            )
        );
        $filterCriteria = new ZmartFilterCriteria("name", array(), 3, 50);
        $filterCriteria->addGroupBy('brand');
        $filterCriteria->addGroupBy('store');
        $filterCriteria->addOrderBy("id", ZmartFilterCriteria::ORDER_DESC);
        $filterCriteria->addOrderBy("name", ZmartFilterCriteria::ORDER_ASC);
        $filterCriteria->addAggregateFunction("BRAND_AVG_PRICE", ZmartFilterCriteria::AVG, 'price');
        $filterCriteria->addAggregateFunction("BRAND_MIN_PRICE", ZmartFilterCriteria::MIN, 'price');
        $filterCriteria->addAggregateFunction("BRAND_MAX_PRICE", ZmartFilterCriteria::MAX, 'price');
        $filterCriteria->addCondition("pow2", ZmartFilterCriteria::GT, 64);
        $filterCriteria->setPostConditions($postConditions);
        $query = $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
        $expectedQuery = array(
            array(
                '$match' => array(
                    'pow2' => array(
                        '$gt' => 64
                    )
                )
            ),
            array(
                '$group' => array(
                    '_id' => array(
                        'brand' => '$brand',
                        'store' => '$store'
                    ),
                    'BRAND_AVG_PRICE' => array(
                        '$avg' => '$price'
                    ),
                    'BRAND_MIN_PRICE' => array(
                        '$min' => '$price'
                    ),
                    'BRAND_MAX_PRICE' => array(
                        '$max' => '$price'
                    )
                )
            ),
            array(
                '$match' => array(
                    'id' => array(
                        '$gt' => 31),
                    'desc' => array(
                        '$eq' => 'sixty four'
                    ),
                    'pow2' => array(
                        '$eq' => 128
                    )
                )
            ),
            array(
                '$sort' => array(
                    'id' => -1,
                    'name' => 1
                )
            ),
            array(
                '$skip' => 100
            ),
            array(
                '$limit' => 50
            )
        );
        $this->assertEquals($expectedQuery, $query);
    }

    public function testInvalidAggregateFunction() {
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 0);
        $filterCriteria->addGroupBy('brand');
        $aggregateFunction = array(
            new AggregateFunction("Fail", ZmartFilterCriteria::AVG, new FilterCriteria('a'))
        );
        $filterCriteria->setAggregateFunctions($aggregateFunction);
        $this->setExpectedException('\Zeedhi\Zmart\QueryBuilders\Mongo\Exception', 'Invalid content type: Zeedhi\Framework\DataSource\FilterCriteria. Types accepteds: \Zeedhi\Zmart\Util\MathExpression, String and Number');
        $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
    }

    public function testInvalidAggregateFunctionArrayExpect() {
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 0);
        $filterCriteria->addGroupBy('brand');
        $aggregateFunction = array(
            new AggregateFunction("Fail", ZmartFilterCriteria::AVG, array(
                "label" => "BRAND_COUNT_PRICES",
                "operator" => ZmartFilterCriteria::COUNT,
                "columnName" => 'price'
            ))
        );
        $filterCriteria->setAggregateFunctions($aggregateFunction);
        $this->setExpectedException('\Zeedhi\Zmart\QueryBuilders\Mongo\Exception', 'Invalid content type: array. Types accepteds: \Zeedhi\Zmart\Util\MathExpression, String and Number');
        $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
    }

    public function testAggregateWithExpressionSum() {
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 0);
        $filterCriteria->addGroupBy('brand');
        $filterCriteria->addAggregateFunction("BRAND_SUM_PRICE", ZmartFilterCriteria::SUM, new MathExpression('price', MathExpression::ADD, 100));
        $query = $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
        $expectedQuery = array(array('$group' => array('_id' => '$brand', 'BRAND_SUM_PRICE' => array('$sum' => array('$add' => array('$price', 100))))));
        $this->assertEquals($expectedQuery, $query);
    }

    public function testAggregateWithExpressionSub() {
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 0);
        $filterCriteria->addGroupBy('brand');
        $filterCriteria->addAggregateFunction("BRAND_SUM_PRICE", ZmartFilterCriteria::SUM, new MathExpression('price', MathExpression::SUB, 100));
        $query = $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
        $expectedQuery = array(array('$group' => array('_id' => '$brand', 'BRAND_SUM_PRICE' => array('$sum' => array('$subtract' => array('$price', 100))))));
        $this->assertEquals($expectedQuery, $query);
    }

    public function testAggregateWithExpressionMul() {
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 0);
        $filterCriteria->addGroupBy('brand');
        $filterCriteria->addAggregateFunction("BRAND_SUM_PRICE", ZmartFilterCriteria::SUM, new MathExpression('price', MathExpression::MUL, 100));
        $query = $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
        $expectedQuery = array(array('$group' => array('_id' => '$brand', 'BRAND_SUM_PRICE' => array('$sum' => array('$multiply' => array('$price', 100))))));
        $this->assertEquals($expectedQuery, $query);
    }

    public function testAggregateWithExpressionDiv() {
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 0);
        $filterCriteria->addGroupBy('brand');
        $filterCriteria->addAggregateFunction("BRAND_SUM_PRICE", ZmartFilterCriteria::SUM, new MathExpression('price', MathExpression::DIV, 100));
        $query = $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
        $expectedQuery = array(array('$group' => array('_id' => '$brand', 'BRAND_SUM_PRICE' => array('$sum' => array('$divide' => array('$price', 100))))));
        $this->assertEquals($expectedQuery, $query);
    }

    public function testAggregateWithNumber() {
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 0);
        $filterCriteria->addGroupBy('brand');
        $filterCriteria->addAggregateFunction("BRAND_SUM_PRICE", ZmartFilterCriteria::SUM, 100);
        $query = $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
        $expectedQuery = array(array('$group' => array('_id' => '$brand', 'BRAND_SUM_PRICE' => array('$sum' => 100))));
        $this->assertEquals($expectedQuery, $query);
    }

    public function testAllActualFunctionsWithExpression() { // Aggregate,Previous Where, Where, Group, Sort, Skip with a MathExpression
        $postConditions = array(
            array(
                'columnName' => 'id',
                'operator' => ZmartFilterCriteria::GT,
                'value' => 31
            ),
            array(
                'columnName' => 'desc',
                'operator' => ZmartFilterCriteria::EQ,
                'value' => 'sixty four'
            ),
            array(
                'columnName' => 'pow2',
                'operator' => ZmartFilterCriteria::EQ,
                'value' => 128
            )
        );
        $filterCriteria = new ZmartFilterCriteria("name", array(), 3, 50);
        $filterCriteria->addGroupBy('brand');
        $filterCriteria->addGroupBy('store');
        $filterCriteria->addOrderBy("id", ZmartFilterCriteria::ORDER_DESC);
        $filterCriteria->addOrderBy("name", ZmartFilterCriteria::ORDER_ASC);
        $filterCriteria->addAggregateFunction("BRAND_AVG_PRICE", ZmartFilterCriteria::AVG, 'price');
        $filterCriteria->addAggregateFunction("BRAND_MIN_PRICE", ZmartFilterCriteria::MIN, 'price');
        $filterCriteria->addAggregateFunction("BRAND_MAX_PRICE", ZmartFilterCriteria::MAX, new MathExpression('price', MathExpression::SUB, 'descount'));
        $filterCriteria->addCondition("pow2", ZmartFilterCriteria::GT, 64);
        $filterCriteria->setPostConditions($postConditions);
        $query = $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
        $expectedQuery = array(
            array(
                '$match' => array(
                    'pow2' => array(
                        '$gt' => 64
                    )
                )
            ),
            array(
                '$group' => array(
                    '_id' => array(
                        'brand' => '$brand',
                        'store' => '$store'
                    ),
                    'BRAND_AVG_PRICE' => array(
                        '$avg' => '$price'
                    ),
                    'BRAND_MIN_PRICE' => array(
                        '$min' => '$price'
                    ),
                    'BRAND_MAX_PRICE' => array(
                        '$max' => array('$subtract' => array('$price', '$descount'))
                    )
                )
            ),
            array(
                '$match' => array(
                    'id' => array(
                        '$gt' => 31),
                    'desc' => array(
                        '$eq' => 'sixty four'
                    ),
                    'pow2' => array(
                        '$eq' => 128
                    )
                )
            ),
            array(
                '$sort' => array(
                    'id' => -1,
                    'name' => 1
                )
            ),
            array(
                '$skip' => 100
            ),
            array(
                '$limit' => 50
            )
        );
        $this->assertEquals($expectedQuery, $query);
    }

    public function testMathExpression() {
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 0);
        $filterCriteria->addMathExpression('test', 100, MathExpression::DIV, 2);
        $query = $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
        $expectedQuery = array(array('$project' => array('test' => array('$divide' => array(100, 2)))));
        $this->assertEquals($expectedQuery, $query);
    }

    public function testMathExpressionInnerMath() {
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 0);
        $secondExpression = new MathExpression('price', MathExpression::ADD, 20);
        $filterCriteria->addMathExpression('test', 100, MathExpression::DIV, $secondExpression);
        $query = $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
        $expectedQuery = array(array('$project' => array('test' => array('$divide' => array(100, array('$add' => array('$price', 20)))))));
        $this->assertEquals($expectedQuery, $query);
    }

    public function testMathExpressionInnerMathJustColumns() {
        $filterCriteria = new ZmartFilterCriteria("name", array(), null, 0);
        $secondExpression = new MathExpression('price', MathExpression::ADD, 'iWant');
        $filterCriteria->addMathExpression('test', 'sumPrice', MathExpression::DIV, $secondExpression);
        $query = $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
        $expectedQuery = array(array('$project' => array('test' => array('$divide' => array('$sumPrice', array('$add' => array('$price', '$iWant')))))));
        $this->assertEquals($expectedQuery, $query);
    }

    public function testProjectWithOtherFunctions() { // Aggregate,Previous Where, Where, Group, Sort, Skip and Project with a MathExpression
        $postConditions = array(
            array(
                'columnName' => 'id',
                'operator' => ZmartFilterCriteria::GT,
                'value' => 31
            ),
            array(
                'columnName' => 'desc',
                'operator' => ZmartFilterCriteria::EQ,
                'value' => 'sixty four'
            ),
            array(
                'columnName' => 'pow2',
                'operator' => ZmartFilterCriteria::EQ,
                'value' => 128
            )
        );
        $filterCriteria = new ZmartFilterCriteria("name", array(), 3, 50);
        $filterCriteria->addGroupBy('brand');
        $filterCriteria->addGroupBy('store');
        $filterCriteria->addOrderBy("id", ZmartFilterCriteria::ORDER_DESC);
        $filterCriteria->addOrderBy("name", ZmartFilterCriteria::ORDER_ASC);
        $filterCriteria->addAggregateFunction("BRAND_AVG_PRICE", ZmartFilterCriteria::AVG, 'price');
        $filterCriteria->addAggregateFunction("BRAND_MIN_PRICE", ZmartFilterCriteria::MIN, 'price');
        $filterCriteria->addAggregateFunction("BRAND_MAX_PRICE", ZmartFilterCriteria::MAX, new MathExpression('price', MathExpression::SUB, 'descount'));
        $filterCriteria->setMathExpressions(array(
            array(
                'label' => "maxLessMin",
                'mathExpression' => new MathExpression("BRAND_MAX_PRICE", MathExpression::SUB, "BRAND_MIN_PRICE")
            )
        ));
        $filterCriteria->addCondition("pow2", ZmartFilterCriteria::GT, 64);
        $filterCriteria->setPostConditions($postConditions);
        $query = $this->queryBuilder->buildQueryFromFilterCriteria($filterCriteria);
        $expectedQuery = array(
            array(
                '$match' => array(
                    'pow2' => array(
                        '$gt' => 64
                    )
                )
            ),
            array(
                '$group' => array(
                    '_id' => array(
                        'brand' => '$brand',
                        'store' => '$store'
                    ),
                    'BRAND_AVG_PRICE' => array(
                        '$avg' => '$price'
                    ),
                    'BRAND_MIN_PRICE' => array(
                        '$min' => '$price'
                    ),
                    'BRAND_MAX_PRICE' => array(
                        '$max' => array('$subtract' => array('$price', '$descount'))
                    )
                )
            ),
            array(
                '$project' => array(
                    'maxLessMin' => array(
                        '$subtract' => array(
                            '$BRAND_MAX_PRICE',
                            '$BRAND_MIN_PRICE'
                        )
                    ),
                    'BRAND_AVG_PRICE' => '$BRAND_AVG_PRICE',
                    'BRAND_MIN_PRICE' => '$BRAND_MIN_PRICE',
                    'BRAND_MAX_PRICE' => '$BRAND_MAX_PRICE'
                )
            ),
            array(
                '$match' => array(
                    'id' => array(
                        '$gt' => 31),
                    'desc' => array(
                        '$eq' => 'sixty four'
                    ),
                    'pow2' => array(
                        '$eq' => 128
                    )
                )
            ),
            array(
                '$sort' => array(
                    'id' => -1,
                    'name' => 1
                )
            ),
            array(
                '$skip' => 100
            ),
            array(
                '$limit' => 50
            )
        );
        $this->assertEquals($expectedQuery, $query);
    }

}